#ifndef HTOR_SETS_HPP
#define HTOR_SETS_HPP

#include <vector>
#include "Graph.hpp"

class HtorSets {
	
	private:
	
	public:

	std::vector< std::pair<Node*,Node*> > Matches;
	std::vector< std::pair<Node*,Node*> > Dependencies;
	std::vector<Node*> Sends;
	std::vector<Node*> Recvs;
	std::vector<Node*> Localops;

	void clear(void) {
		Matches.clear();
		Dependencies.clear();
		Sends.clear();
		Recvs.clear();
		Localops.clear();
	}

	void addNodeAndDeps(Node* n) {
		if (n->getType() == T_SEND) this->addSend(n);
		if (n->getType() == T_RECV) this->addRecv(n);
		//std::map<int, Node*>::iterator dit = n->getNodesWhoDependOnUs()->begin();
		std::vector<Node*>::iterator dit = n->getNodesWhoDependOnUs()->begin();
		while (dit != n->getNodesWhoDependOnUs()->end()) {
			//addDependency(n, dit->second);
			addDependency(n, *dit);
			++dit;
		}	
	}

	void addSend(Node* n) {
		this->Sends.push_back(n);
	}

	void addRecv(Node* n) {
		this->Recvs.push_back(n);
	}
	
	void addDependency(Node* first, Node* next) {
		this->Dependencies.push_back(std::pair<Node*,Node*>(first, next));		
	}

	void addMatch(Node* n1, Node* n2) {
		if (n1->getType() == T_SEND) this->Matches.push_back(std::pair<Node*,Node*>(n1, n2));		
		if (n2->getType() == T_SEND) this->Matches.push_back(std::pair<Node*,Node*>(n2, n1));		
	}

	void printSends() {
		std::cout << "Set of sends:" << std::endl;
		std::vector<Node*>::iterator it = this->Sends.begin();
		while (it!=this->Sends.end()) {
			(*it)->printNode();
			++it;
		}
		std::cout << std::endl;
	}

	void printRecvs() {
		std::cout << "Set of recvs:" << std::endl;
		std::vector<Node*>::iterator it = this->Recvs.begin();
		while (it!=this->Recvs.end()) {
			(*it)->printNode();
			++it;
		}		
		std::cout << std::endl;
	}

	void printDeps() {
		std::cout << "Set of dependencies:" << std::endl;
		std::vector< std::pair<Node*,Node*> >::iterator it = this->Dependencies.begin();
		while (it!=this->Dependencies.end()) {
			std::cout << "[" << std::endl;
			std::cout << "\t";
			it->first->printNode();
			std::cout << "\t";
			it->second->printNode();
			++it;
			std::cout << "]" << std::endl;
		}
		std::cout << std::endl;
	}

	void printMatches() {
		std::cout << "Set of matches:" << std::endl;
		std::vector< std::pair<Node*,Node*> >::iterator it = this->Matches.begin();
		while (it!=this->Matches.end()) {
			std::cout << "[" << std::endl;
			std::cout << "\t";
			it->first->printNode();
			std::cout << "\t";
			it->second->printNode();
			++it;
			std::cout << "]" << std::endl;
		}		
		std::cout << std::endl;
	}

	void printMatchesAsDot() {
		std::vector< std::pair<Node*,Node*> >::iterator it = this->Matches.begin();
		while (it!=this->Matches.end()) {
			std::cout << "\"" <<  it->first->toString() << "\" -> \"" << it->second->toString() << "\" [color=green]" << std::endl;
			++it;
		}		
	}

	void printDepsAsDot() {
		std::vector< std::pair<Node*,Node*> >::iterator it = this->Dependencies.begin();
		while (it!=this->Dependencies.end()) {
			std::cout << "\"" << it->second->toString() << "\" -> \"" << it->first->toString() << "\" [color=blue]" << std::endl;
			++it;
		}
	}

	void printSets() {
		std::cout << std::endl << "Htor Sets (owner:type:peer:addr:size:tag)" << std::endl;
		this->printSends();		
		this->printRecvs();		
		this->printDeps();		
		this->printMatches();		
	}

	void printGlobalGraphAsDot() {
		std::cout << "digraph {" << std::endl;
		this->printMatchesAsDot();		
		this->printDepsAsDot();
		std::cout << "}" << std::endl;		
	}
	
};

#endif

