#include <map>
#include <queue>
#include "Graph.hpp"
//#include "Matcher.hpp"
#include "HtorSets.hpp"
// #include "ActiveQueue.hpp"

typedef struct MNode {
	int owner;
	int peer;
	int size;
	int tag;
	char type;
} MNode;

MNode* _tmpmnode = new MNode;

struct Comparator {
	bool operator()(const MNode a, const MNode b) const {
		if (a.type != b.type) return a.type < b.type;
		if (a.owner != b.owner) return a.owner < b.owner;
		if (a.peer != b.peer) return a.peer < b.peer;
		if (a.size != b.size) return a.size < b.size;
		if (a.tag != b.tag) return a.tag < b.tag;
		return false;
	}
};

typedef std::multimap </*node data*/ MNode, /*real node*/ Node*, Comparator> ActiveQueue;


static inline void resolve_deps(Node* node, std::queue < Node* >* FreeQueue) {
	
	//std::map<int,Node*>* dependents = node->getNodesWhoDependOnUs();
	std::vector<Node*>* dependents = node->getNodesWhoDependOnUs();
    //for (std::map<int,Node*>::iterator dit = dependents->begin(); dit!=dependents->end(); dit++) {
    for (std::vector<Node*>::iterator dit = dependents->begin(); dit!=dependents->end(); dit++) {
		(*dit)->decrDepCnt();
		if ((*dit)->getDepCnt()==0) {
			FreeQueue->push(*dit);
		} 
	}
}

static inline Node* find_match(Node* n, ActiveQueue* aq) {
	
	char type = n->getType();
	if (type == T_SEND) type = T_RECV;
	else if (type == T_RECV) type = T_SEND;
	
	/*                             owner and peer are swapped here!  */
	MNode* x = _tmpmnode;
	x->type = type;
	x->owner = n->getPeer();
	x->peer = n->getOwner();
	x->size = n->getSize();
	x->tag = n->getTag();
	ActiveQueue::iterator it;
	it = aq->find(*x);
	
	if (it == aq->end()) {
		return ((Node*) NULL);
	}
	else {
		return it->second;
	}
}

static inline void insert(Node* n, ActiveQueue* aq) {
	MNode* x = _tmpmnode;
	x->type = n->getType();
	x->owner = n->getOwner();
	x->peer = n->getPeer();
	x->size = n->getSize();
	x->tag = n->getTag();
	aq->insert(std::make_pair(*x, n));
}

static inline void remove(Node* n, ActiveQueue* aq) {
	
	MNode* x = _tmpmnode;
	x->type = n->getType();
	x->owner = n->getOwner();
	x->peer = n->getPeer();
	x->size = n->getSize();
	x->tag = n->getTag();
	std::pair < ActiveQueue::iterator, ActiveQueue::iterator > ret;
	ActiveQueue::iterator it;
	ret = aq->equal_range(*x);
	for (it=ret.first; it!=ret.second; ++it) {   // TODO this is different from pseudocode
		if (it->second == n) {
			aq->erase(it);
			return;	
		}
	}

}


HtorSets* Matcher(SetOfGraphs* LocGraphs) {

	/* see algorithm description on how these two are used (hint: its AQ_i and FQ) */			
	std::map </*rank*/ int, ActiveQueue*> SetOfActiveQueues;
	std::queue < Node* > FreeQueue;
	
	/* what we output */
	HtorSets* hs = new HtorSets();

	/* initialize FQ */
	for (SetOfGraphs::iterator it=LocGraphs->begin(); it!=LocGraphs->end(); it++) {
		std::map<int, Node*>* ans = it->second->getActiveNodes();
		std::map<int, Node*>::iterator iter = ans->begin();
		while (iter != ans->end()) {
			FreeQueue.push(iter->second);
			iter++;		
		}
	}
	
	/* Initialize AQ_i */	
	for (SetOfGraphs::iterator it=LocGraphs->begin(); it!=LocGraphs->end(); it++) {
		ActiveQueue* aq = new ActiveQueue;
		SetOfActiveQueues.insert(std::pair<int, ActiveQueue*>(it->first, aq));
	}

	/* the main loop of the matching alg */
	while (!FreeQueue.empty()) {
		/* a <- pop(FQ)*/
		Node* a = FreeQueue.front();
		FreeQueue.pop(); // pop doesn't return the value, WTF!?

		if ((a->getType() == T_OP) || (a->getType() == T_OPVEC)) {
			resolve_deps(a, &FreeQueue);
			hs->addNodeAndDeps(a);
			continue;		
		}
		
		Node* y = find_match(a, SetOfActiveQueues[a->getPeer()]);

		if (y == NULL) {
			insert(a, SetOfActiveQueues[a->getOwner()]);
		}
		else {
			remove(y, SetOfActiveQueues[a->getPeer()]);
			resolve_deps(a, &FreeQueue);
			resolve_deps(y, &FreeQueue);
			hs->addMatch(a,y);	// add_match() in pseudocode
			hs->addNodeAndDeps(a);
			hs->addNodeAndDeps(y);
			a->setMatchingNode(y);
			y->setMatchingNode(a);
		}
	}

	//hs->printSets();

	std::map </*rank*/ int, ActiveQueue*>::iterator it = SetOfActiveQueues.begin();
	while (it != SetOfActiveQueues.end()) {
		delete(it->second);
		it++;
	}
	SetOfActiveQueues.clear();

	return hs;


}

