/**
 * @file   cDAG.hpp
 * @Author
 * @date
 * @brief contains interface description for cDAG
 *
 * This file describes the interface of the cDAG library. It is the only file that should be included
 * by programs using cDAG.
 */

#include "cDAG_Types.hpp"	// internal types

typedef int cDAG_Vertex;
typedef int cDAG_Opcode;
const cDAG_Vertex cDAG_VERTEX_NULL=-1;
typedef cDAG_MemType_enum cDAG_MemType;
typedef cDAG_Handle_struct* cDAG_Handle;
typedef cDAG_Schedule_struct cDAG_Schedule;


int cDAG_Init();
int cDAG_IsInitialized();
int cDAG_Finalize();
int cDAG_IsFinalized();
int cDAG_GroupSize();
int cDAG_MyRank();
cDAG_Graph cDAG_Create_graph();
int cDAG_FreeGraph(cDAG_Graph g);
cDAG_Vertex cDAG_Send(cDAG_Graph graph, void* buf, int count, int dest, cDAG_MemType mem=cDAG_USERSPACE);
cDAG_Vertex cDAG_Recv(cDAG_Graph graph, void* buf, int count, int source, cDAG_MemType mem=cDAG_USERSPACE);
cDAG_Opcode cDAG_RegisterOp(cDAG_Graph graph, void (*func)(void*, void*), int argsize);
cDAG_Vertex cDAG_Op(cDAG_Graph graph, cDAG_Opcode op, void* myargs);
void cDAG_Requires(cDAG_Graph g, cDAG_Vertex prereq, cDAG_Vertex target);
int cDAG_AllocateScratchpad(cDAG_Graph graph, size_t bytes);
cDAG_Schedule cDAG_Compile(cDAG_Graph graph);
int cDAG_IsScheduleValid(cDAG_Schedule sched);
size_t cDAG_MakeScheduleReusable(cDAG_Schedule sched);
int cDAG_FreeSchedule(cDAG_Schedule sched);
cDAG_Handle cDAG_Run(cDAG_Schedule sched);
int cDAG_Test(cDAG_Handle handle);
int cDAG_Wait(cDAG_Handle handle);


